@@ -5,7 +5,7 @@ use warnings;
 use utf8;
 use Regexp::Grammars;
 
-our $VERSION = '5.005';
+our $VERSION = '5.006';
 
 sub new {
     my ($class) = @_;
@@ -226,10 +226,10 @@ sub PT::FunctionList::X {
 }
 
 sub PT::Constructor::X {
-        my ($class, $className) = @_;
+    my ($class, $className) = @_;
 
-        my $constructor = $class->{''};
-        return $constructor;
+    my $constructor = $class->{''};
+    return $constructor;
 }
 
 sub PT::Object::X {
@@ -901,7 +901,7 @@ sub PT::Assignment::X {
 sub PT::AccessorAssignment::X {
     my ($class, $className) = @_;
 
-    my $variableName = $class->{HashKeyStringValue}->X($className);
+    my $variableName = $class->{String}->X($className);
     my $rhs = $class->{RHS}->X($className);
 
     my $accessorAssignment  = '$class->{"' . $className . '"}->{"'. $variableName .'"} = ' . $rhs .';';
@@ -977,7 +977,7 @@ sub PT::ObjectFunctionCall::X {
 
 sub PT::ClassAccessor::X {
     my ($class, $className) = @_;
-    my $variableName = $class->{HashKeyStringValue}->X($className);
+    my $variableName = $class->{String}->X($className);
 
     my $classAccessor = '$class->{"' . $className . '"}->{"'. $variableName .'"}';
     return $classAccessor;
@@ -1047,10 +1047,20 @@ sub PT::ArrayElement::X {
 
 sub PT::ArrayAccess::X {
     my ($class, $className) = @_;
-    my $number = $class->{Number}->X($className);
+    my $number = $class->{ArrayKey}->X($className);
     return $number;
 }
 
+sub PT::ArrayKey::X {
+    my ($class, $className) = @_;
+    return (       $class->{Number}
+                || $class->{ScalarVariable}
+                || $class->{ArrayElement}
+                || $class->{HashElement}
+                || $class->{FunctionReturn}
+                || $class->{ClassFunctionReturn} )->X($className);
+}
+
 sub PT::ArrayName::X {
     my ($class, $className) = @_;
     my $arrayName = $class->{''};
@@ -1088,8 +1098,13 @@ sub PT::HashName::X {
 
 sub PT::HashKey::X {
     my ($class, $className) = @_;
-    return (       $class->{HashKeyString}
-                || $class->{HashKeyNumber} )->X($className);
+    return (       $class->{String}
+                || $class->{Number}
+                || $class->{ScalarVariable}
+                || $class->{ArrayElement}
+                || $class->{HashElement}
+                || $class->{FunctionReturn}
+                || $class->{ClassFunctionReturn} )->X($className);
 }
 
 sub PT::HashKeyString::X {
@@ -1361,8 +1376,10 @@ my $parser = qr {
 
     <objrule:  PT::For>                        <TokenFor> <Var> <VariableName> <LParen> <ForRange> <RParen> <CodeBlock>
     <objrule:  PT::ForRange>                   <LowerRange> <Dot><Dot><Dot> <UpperRange>
+
     <objrule:  PT::LowerRange>                 <Number> | <VariableName> | <ArrayElement> | <HashElement>
                                                 | <ClassAccessor> | <ClassFunctionReturn> | <FunctionReturn>
+
     <objrule:  PT::UpperRange>                 <Number> | <VariableName> | <ArrayElement> | <HashElement>
                                                 | <ClassAccessor> | <ClassFunctionReturn> | <FunctionReturn>
 
@@ -1372,8 +1389,10 @@ my $parser = qr {
     <objrule:  PT::BoolExpression>             <[BooleanExpression]>+ % <[BoolOperator]>
     <objrule:  PT::BooleanExpression>          <BoolOperands> <BoolOperatorExpression>?
     <objrule:  PT::BoolOperatorExpression>     <BoolOperator> <BoolOperands>
+
     <objrule:  PT::BoolOperands>               <RealNumber> | <String> | <ScalarVariable> | <ArrayElement> | <HashElement>
                                                | <ClassAccessor> | <ClassFunctionReturn> | <FunctionReturn> | <EmbedBlock>
+
     <objrule:  PT::BoolOperator>               <GreaterThan> | <LessThan> | <Equals> | <GreaterThanEquals> | <LessThanEquals>
                                                | <StringEquals> | <StringNotEquals> | <NotEqulas> | <LogicalAnd> | <LogicalOr>
                                                | <EmbedBlock>
@@ -1387,9 +1406,7 @@ my $parser = qr {
     <objrule:  PT::ClassFunctionCall>          <TokenClass> <Dot> <FunctionName> <LParen> <Parameters>? <RParen> <SemiColon>
 
     <objrule:  PT::ObjectCall>                 <ObjectFunctionCall> <SemiColon>
-
     <objrule:  PT::VariableDeclaration>        <ArrayDeclaration> | <HashDeclaration> | <ScalarDeclaration>
-
     <objrule:  PT::ScalarDeclaration>          <Var> <VariableName> <Equal> <Value> <SemiColon>
 
     <objtoken: PT::Var>                        var
@@ -1405,6 +1422,7 @@ my $parser = qr {
     <objrule:  PT::ArrayDeclaration>           <Var> <VariableName> <Equal> <ArrayList> <SemiColon>
     <objrule:  PT::ArrayList>                  <LBracket> <ListElements> <RBracket>
     <objrule:  PT::ListElements>               .{0} | <[ListElement]>+ % <Comma>
+
     <objrule:  PT::ListElement>                <RealNumber> | <String> | <ClassFunctionReturn> | <FunctionReturn>
                                                 | <ArrayElement> | <HashElement> | <ArrayList> | <HashRef>
                                                 | <VariableName> | <EmbedBlock>
@@ -1413,6 +1431,7 @@ my $parser = qr {
     <objrule:  PT::HashRef>                    <LBrace> <KeyValuePairs> <RBrace>
     <objrule:  PT::KeyValuePairs>              .{0} | <[KeyValue]>+ % <Comma>
     <objrule:  PT::KeyValue>                   <PairKey> <Colon> <PairValue>
+
     <objrule:  PT::PairKey>                    <Number> | <String> | <ClassFunctionReturn> | <FunctionReturn>
                                                 | <VariableName> | <EmbedBlock>
 
@@ -1438,21 +1457,21 @@ my $parser = qr {
     <objrule:  PT::FunctionReturn>             <FunctionName> <LParen> <Parameters>? <RParen>
 
     <objrule:  PT::ArrayElement>               <ArrayName> <[ArrayAccess]>+
-    <objrule:  PT::ArrayAccess>                <LBracket> <Number> <RBracket>
+    <objrule:  PT::ArrayAccess>                <LBracket> <ArrayKey> <RBracket>
+    <objrule:  PT::ArrayKey>                   <Number> | <ScalarVariable> | <ArrayElement>
+                                               | <HashElement> | <FunctionReturn> | <ClassFunctionReturn>
     <objrule:  PT::ArrayName>                  [a-zA-Z]+?
 
     <objrule:  PT::HashElement>                <HashName> <[HashAccess]>+
     <objrule:  PT::HashAccess>                 <LBrace> <HashKey> <RBrace>
     <objtoken: PT::HashName>                   [a-zA-Z]+?
-    <objrule:  PT::HashKey>                    <HashKeyString> | <HashKeyNumber>
-    <objrule:  PT::HashKeyString>              <LQuote> <HashKeyStringValue> <RQuote>
-    <objtoken: PT::HashKeyStringValue>         [a-zA-Z]+?
-    <objtoken: PT::HashKeyNumber>              [0-9]+?
+    <objrule:  PT::HashKey>                    <String> | <Number> | <ScalarVariable> | <ArrayElement>
+                                               | <HashElement> | <FunctionReturn> | <ClassFunctionReturn>
 
     <objrule:  PT::STDIN>                      <LessThan>  <TokenSTDIN> <GreaterThan>
 
-    <objrule:  PT::AccessorAssignment>         <TokenClass> <Dot> <HashKeyStringValue> <Equal> <RHS> <SemiColon>
-    <objrule:  PT::ClassAccessor>              <TokenClass> <Dot> <HashKeyStringValue>
+    <objrule:  PT::AccessorAssignment>         <TokenClass> <Dot> <String> <Equal> <RHS> <SemiColon>
+    <objrule:  PT::ClassAccessor>              <TokenClass> <Dot> <String>
     <objrule:  PT::ClassFunctionReturn>        <TokenClass> <Dot> <FunctionName> <LParen> <Parameters>? <RParen>
 
     <objrule:  PT::ArrayAssignment>            <ArrayElement> <Equal> <RHS> <SemiColon>
